import { ComponentFixture, TestBed } from '@angular/core/testing';

import { ActivityCardComponent } from './activity-card.component';
import {ActivitiesService} from '../activities.service';
import {DataCardService} from '../../api/data-card.service';
import SpyObj = jasmine.SpyObj;
import {createDataCardServiceSpy} from '../../api/data-card.service.spec';
import {AngularMaterialModule} from '../../modules/angular-material.module';

describe('ActivityCardComponent', () => {
  let component: ActivityCardComponent;
  let fixture: ComponentFixture<ActivityCardComponent>;

  let activitiesServiceStub: ActivitiesService;
  let dataCardServiceStub: SpyObj<DataCardService>;

  beforeEach(async () => {
    activitiesServiceStub = {
      emitOpenActivityModalEvent: () => {},
      cancelActivity: () => {},
      emitOpenNoteModalEvent: () => {},
      emitOpenMeasurementsModalEvent: () => {},
    } as any;

    dataCardServiceStub = createDataCardServiceSpy();

    await TestBed.configureTestingModule({
      declarations: [ ActivityCardComponent ], imports: [AngularMaterialModule],
      providers: [
        { provide: ActivitiesService, useValue: activitiesServiceStub },
        { provide: DataCardService, useValue: dataCardServiceStub }
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(ActivityCardComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('when there is no activity set', () => {
    beforeEach(() => {
      component.activity = undefined;
    });

    it('does not call emitOpenActivityModalEvent', () => {
      const spy = spyOn(activitiesServiceStub, 'emitOpenActivityModalEvent');
      component.editActivity();
      expect(spy).toHaveBeenCalledTimes(0);
    });

    it('does not call cancelActivity', () => {
      const spy = spyOn(activitiesServiceStub, 'cancelActivity');
      component.cancelActivity();
      expect(spy).toHaveBeenCalledTimes(0);
    });

    it('does not call emitOpenNoteModalEvent', () => {
      const spy = spyOn(activitiesServiceStub, 'emitOpenNoteModalEvent');
      component.addOrEditNote();
      expect(spy).toHaveBeenCalledTimes(0);
    });

    it('does not call emitOpenMeasurementsModal', () => {
      const spy = spyOn(activitiesServiceStub, 'emitOpenMeasurementsModalEvent');
      component.addOrEditMeasurements();
      expect(spy).toHaveBeenCalledTimes(0);
    });
  });

  describe('when there is an activity set', () => {
    beforeEach(() => {
      component.activity = { id: '12345', status: 'expected', title: 'whatever'};
    });

    it('calls emitOpenActivityModalEvent', () => {
      const spy = spyOn(activitiesServiceStub, 'emitOpenActivityModalEvent');
      component.editActivity();
      expect(spy).toHaveBeenCalledTimes(1);
    });

    it('calls cancelActivity', () => {
      const spy = spyOn(activitiesServiceStub, 'cancelActivity');
      component.cancelActivity();
      expect(spy).toHaveBeenCalledTimes(1);
    });

    it('calls emitOpenNoteModalEvent', () => {
      const spy = spyOn(activitiesServiceStub, 'emitOpenNoteModalEvent');
      component.addOrEditNote();
      expect(spy).toHaveBeenCalledTimes(1);
    });

    it('calls emitOpenMeasurementsModal', () => {
      const spy = spyOn(activitiesServiceStub, 'emitOpenMeasurementsModalEvent');
      component.addOrEditMeasurements();
      expect(spy).toHaveBeenCalledTimes(1);
    });
  });

  it('does not have a white statusColor when status = planned', () => {
    expect(component.getActivityStatusColor('planned')).not.toBe('#FFF');
  });

  it('does not have a white statusColor when status = expected', () => {
    expect(component.getActivityStatusColor('expected')).not.toBe('#FFF');
  });

  it('does not have a white statusColor when status = finished', () => {
    expect(component.getActivityStatusColor('finished')).not.toBe('#FFF');
  });

  it('does have a white statusColor when status is not well defined', () => {
    expect(component.getActivityStatusColor('whatever')).toBe('#FFF');
  });

  describe('observation related tests', () => {
    beforeEach(() => {
      const observations = Array.from(Array(10).keys()).map(value => {
        if (value % 3 === 0) {
          return {fetus: 'Foster 1', id: value, type: 'weight'};
        } else if (value % 3 === 1) {
          return {fetus: 'Foster 2', id: value, type: 'blood_pressure'};
        }
        return {fetus: undefined, id: value};
      }) as any;
      component.activity = { observations } as any;
      console.log(component.activity);
    });

    it('should filter away nullish fetus names', () => {
      expect(component.getFeti().length).toBe(2);
      expect(component.getFeti().indexOf('Foster 1') > -1).toBeTrue();
      expect(component.getFeti().indexOf('Foster 2') > -1).toBeTrue();
    });

    it('should have regular observations', () => {
      expect(component.hasRegularMeasurements()).toBeTrue();
    });

    it('should not have ultrasound observations', () => {
      expect(component.hasUltraSoundMeasurements()).toBeFalse();
    });
  });
});
