import {ComponentFixture, TestBed} from '@angular/core/testing';

import { ActivityModalComponent } from './activity-modal.component';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {AngularMaterialModule} from '../../modules/angular-material.module';
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';
import {Component, Input} from '@angular/core';
import {Activity, CarePlan} from '@nspop/gm-web-facade-api';
import {Observable, Subject, throwError} from 'rxjs';
import {CarePlanService} from '../../api/care-plan.service';
import {ActivitiesService} from '../activities.service';
import {of} from 'rxjs/internal/observable/of';
import {DataCardService} from '../../api/data-card.service';
import {MatRadioChange} from '@angular/material/radio';

describe('ActivityModalComponent', () => {
  const activitiesServiceStub: ActivitiesService = {} as any;
  let component: ActivityModalComponent;
  let fixture: ComponentFixture<ActivityModalComponent>;
  let carePlanSubjectStub: Subject<CarePlan>;
  let carePlanServiceStub: CarePlanService;

  @Component({selector: 'app-modal', template: '<div><ng-content></ng-content></div>'})
  class ModalComponent {
    @Input() public header = '';
    private isModalVisible = false;
    public setVisible(isVisible: boolean): void {
      this.isModalVisible = isVisible;
    }
  }
  beforeEach(async () => {
    carePlanSubjectStub = new Subject<CarePlan>();
    carePlanServiceStub = {
      carePlan$: of( { id: '123', activities: []}),
      update: () => of( { id: '123'}),
      refresh: () => of(true)
    } as any;
    const dataCardServiceStub = {
      selectedDataCard$: of({})
    };

    await TestBed.configureTestingModule({
      declarations: [ ActivityModalComponent, ModalComponent],
      imports: [
        ReactiveFormsModule,
        FormsModule,
        ReactiveFormsModule,
        AngularMaterialModule,
        BrowserAnimationsModule,
      ],
      providers: [
        {provide: CarePlanService, useValue: carePlanServiceStub},
        {provide: ActivitiesService, useValue: activitiesServiceStub},
        {provide: DataCardService, useValue: dataCardServiceStub}
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(ActivityModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('when creating a new activity', () => {
    it('closes modal if submitting carePlan is a success', () => {
      component.activity = undefined;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: []} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      carePlanSubjectStub.next(null);
      expect(component.isVisible()).toBe(false);
    });
  });

  describe('when editing an activity', () => {
    it('closes modal if submitting carePlan is a success', () => {
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ activity ]} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      carePlanSubjectStub.next(null);
      expect(component.isVisible()).toBe(false);
    });

    it('does not submit if there is no carePlan', () => {
      spyOn(component.carePlanService, 'update');
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = undefined;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
    });

    it('does not submit if there is no carePlan', () => {
      spyOn(component.carePlanService, 'update');
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ activity ]} as any as CarePlan;
      component.dataCard = undefined;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
    });

    it('does not submit if it cannot find the activity', () => {
      spyOn(component.carePlanService, 'update');
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ { id: '2', status: 'expected'} ]} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
    });

    it('does not submit if the form is invalid', () => {
      spyOn(component.carePlanService, 'update');
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ { id: '2', status: 'expected'} ]} as any as CarePlan;
      component.form?.setValue(
        {
          title: '',
          description: '',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
    });
  });

  it('sets editMode = true if there is an activity, when setting Activity', () => {
    expect(component.isEditMode).toBeFalsy();

    const activity = {status: 'expected', id: '1'} as Activity;
    component.setActivity(activity);

    expect(component.isEditMode).toBe(true);
  });

  it('sets editMode = false if there is no activity, when setting Activity', () => {
    const activity = {status: 'expected', id: '1'} as Activity;
    component.setActivity(activity);
    expect(component.isEditMode).toBe(true);

    component.setActivity(undefined);
    expect(component.isEditMode).toBe(false);
  });

  it('should disable the note date field if activity is already conducted', () => {
    const conductedActivity = {
      conductedTime: new Date()
    } as Activity;
    const activityAlreadyConducted = component.isActivityAlreadyConducted(conductedActivity);

    expect(activityAlreadyConducted).toBeTrue();
  });

  it('should NOT disable the note date field if activity has not been conducted', () => {
    const notConductedActivity = { } as Activity;
    const activityNotConducted = component.isActivityAlreadyConducted(notConductedActivity);

    expect(activityNotConducted).toBeFalse();
  });

  describe('activity form validity test', () => {
    it('INVALID when Activity is FINISHED and Note is EMPTY', () => {
      component.form?.setValue(
        {
          title: 'A Title',
          description: 'Hello',
          status: 'finished',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: ''
        });
      expect(component.form?.invalid).toBeTrue();
    });
    it('VALID when Activity is FINISHED and Note is FILLED', () => {
      component.form?.setValue(
        {
          title: 'A Title',
          description: 'Hello',
          status: 'finished',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: 'Test'
        });
      expect(component.form?.valid).toBeTrue();
    });
    it('INVALID when Activity is EXPECTED and fromWeek is NULL', () => {
      component.form?.setValue(
        {
          title: 'A Title',
          description: 'Hello',
          status: 'expected',
          fromWeek: null,
          toWeek: new Date(),
          note: ''
        });
      expect(component.form?.invalid).toBeTrue();
    });
    it('VALID when Activity is EXPECTED and fromWeek is NOW', () => {
      component.form?.setValue(
        {
          title: 'A Title',
          description: 'Hello',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: ''
        });
      expect(component.form?.valid).toBeTrue();
    });
    it('should mark note as dirty when changed', () => {
      expect(component.isNoteDirty).toBeFalse();
      component.onNoteContentChanged();
      expect(component.isNoteDirty).toBeTrue();
    });
    it('should return true when content is empty', () => {
      expect(component.isNoteContentEmpty()).toBeTrue();
      component.form?.controls.note?.setValue('foo bar');
      expect(component.isNoteContentEmpty()).toBeFalse();
    });
    it('should set form values when activity status changes', () => {
      component.statusChanged({value: 'finished'} as MatRadioChange);
      expect(component.form?.controls.note?.enabled).toBeTrue();
    });
    it('should not show error if not is emoty with a finished activity', () => {
      component.form?.controls.status?.setValue('finished');
      component.form?.controls.note?.setValue('foo bar');
      expect(component.shouldShowNoteError()).toBeFalse();
    });
    it('should handle errors when submitting',  () => {
      const errorSpy = spyOn(carePlanServiceStub, 'update')
        .and.throwError('error!');
      component.carePlan = {id: '123', version: 456, activities: []} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });
      try {
        component.submit(component.form, component.formDirective);
      } catch (e) {
        expect(errorSpy).toThrowError('error!');
      }
    });
    it('should not do anything in the event of an undefined form', () => {
      component.form = undefined;
      component.statusChanged({value: 'finished'} as MatRadioChange);
      component.statusChanged({value: 'expected'} as MatRadioChange);
      expect(component.form).toBeUndefined();
    });
  });
});
