import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';
import {of} from 'rxjs';
import {FormsModule, NgForm} from '@angular/forms';
import {map} from 'rxjs/operators';
import {DataCardService} from '../../api/data-card.service';
import {ObservationService} from '../../api/observation.service';
import {MeasurementsModalComponent} from './measurements-modal.component';
import {
  Activity,
  ObservationTable,
  ObservationTableFetusesInner,
  ScanningObservationTable,
  ScanningObservationTableFetusesInner
} from '@nspop/gm-web-facade-api';
import {SimpleChange} from '@angular/core';
import {NoteService} from '../../api/note.service';
import {ModalComponent} from '../../modals/modal/modal.component';
import {AngularMaterialModule} from '../../modules/angular-material.module';
import {UltrasoundModalSectionComponent} from './ultrasound-section/ultrasound-modal-section.component';
import {DatePickerComponent} from '../../date-picker/date-picker.component';
import {TieToActivityModalComponent} from '../tie-to-activity-modal/tie-to-activity-modal.component';
import {RegularModalSectionComponent} from './regular-section/regular-modal-section.component';
import {DropdownComponent} from '../../dropdown/dropdown.component';
import {TextAreaComponent} from '../../text-area/text-area.component';
import {CarePlanService} from '../../api/care-plan.service';
import {BrowserAnimationsModule, NoopAnimationsModule} from '@angular/platform-browser/animations';

function getActivity(): Activity {
  return {title: 'some activity', id: '1234abcd'} as Activity;
}

describe('MeasurementsModalComponent', () => {
  let component: MeasurementsModalComponent;
  let fixture: ComponentFixture<MeasurementsModalComponent>;
  let dataCardServiceStub: DataCardService;
  let observationServiceStub: ObservationService;
  let noteServiceStub: NoteService;
  let carePlanServiceStub: CarePlanService;

  beforeEach(async () => {
    const dummyDataCardResponse = {
      expectedDateOfDeliverySection: {},
      prenatalRiskAssessmentSection: {
        finalRiskAssessment: {
          multiplicityRiskAssessment: {}
        },
      },
      overallAssessmentSection: {prenatalCareAssessment: {numberOfFetuses: 2}}
    };
    dataCardServiceStub = {
      isDataCardClosed: of(false),
      selectedDataCard$: of(dummyDataCardResponse),
      refresh: () => of(),
      update: () => of(dummyDataCardResponse),
    } as any;
    noteServiceStub = {
      createOrUpdateNote: () => of({})
    } as any;
    carePlanServiceStub = {
      activities$: of([])
    } as any;
    observationServiceStub = {
      createUltrasoundObservations: () => of(),
      createRegularObservations: () => of(),
      ultrasoundObservationTable$: of({mother: {rows: [{activityId: '1234-abcd', id: '1', flow: 'afvigende'}]}} as ScanningObservationTable),
      clinicalObservationTable$:  of({mother: {rows: [{activityId: '1234-abcd', id: '1'}]}} as ObservationTable)
    } as any;

    await TestBed.configureTestingModule({
      declarations: [
        MeasurementsModalComponent,
        ModalComponent,
        UltrasoundModalSectionComponent,
        DatePickerComponent,
        TieToActivityModalComponent,
        RegularModalSectionComponent,
        DropdownComponent,
        TextAreaComponent
      ],
      imports: [FormsModule, AngularMaterialModule, NoopAnimationsModule, BrowserAnimationsModule],
      providers: [
        {provide: NoteService, useValue: noteServiceStub},
        {provide: DataCardService, useValue: dataCardServiceStub},
        {provide: ObservationService, useValue: observationServiceStub},
        {provide: CarePlanService, useValue: carePlanServiceStub}
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(MeasurementsModalComponent);
    component = fixture.componentInstance;
    component.observationDataUltrasound = {};
    component.observationDataRegular = {};
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('fetches datacard and resets observationData on init', fakeAsync(() => {
    let accessed = false;
    dataCardServiceStub.selectedDataCard$ = of({multiplePregnancySection: {numberOfFetuses: 3}} as any).pipe(
      map(x => {accessed = true; return x; })
    );

    fixture = TestBed.createComponent(MeasurementsModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();

    component.isOpen = true;
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    fixture.detectChanges();
    component.ngOnInit();
    tick();
    expect(accessed).toBeTrue();
    expect(component.observationDataRegular?.fetuses?.length).toBe(3);
  }));

  it('allows save if activity was picked', fakeAsync(() => {
    component.isOpen = true;
    component.activity = getActivity();
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    const spy = spyOn(observationServiceStub, 'createRegularObservations').and.callThrough();
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(spy).toHaveBeenCalled();
  }));

  it('does not allow save if no activity was picked', fakeAsync(() => {
    component.activity = undefined;
    component.isOpen = true;
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    const spy = spyOn(observationServiceStub, 'createRegularObservations').and.callThrough();
    component.observationDataRegular = {} as any;
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(spy).not.toHaveBeenCalled();
  }));

  it('should set measurementDate when activity is saved', fakeAsync(() => {
    component.isOpen = true;
    component.activity = getActivity();
    component.observationType = 'regular';
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.measurementDate = new Date(2010, 10, 20);
    component.observationType = 'regular';
    component.doSave();
    expect(component?.observationDataRegular?.date).toEqual(component.measurementDate);
  }));

  it('should set offsetDate when offsetData exists on the dataCard', fakeAsync(() => {
    const offsetDate = new Date(2010, 6,  10);
    component.isOpen = true;
    component.activity = getActivity();
    component.observationType = 'regular';
    component.dataCard = {
      expectedDateOfDeliverySection: {
        offsetDate,
      }
    } as any;
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    component.doSave();
    expect(component?.observationDataRegular?.offsetDate).toEqual(offsetDate);
  }));

  it('should not set offsetDate when offsetData does not exists on the dataCard', fakeAsync( () => {
    component.isOpen = true;
    component.activity = getActivity();
    component.observationType = 'regular';
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    component.doSave();
    expect(component?.observationDataRegular?.offsetDate).toBeUndefined();
  }));


  it('just creates observation when in "regular" mode', fakeAsync(() => {
    component.isOpen = true;
    component.activity = getActivity();
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    component.observationType = 'regular';
    const regularObservationSpy = spyOn(observationServiceStub, 'createRegularObservations').and.callThrough();
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(regularObservationSpy).toHaveBeenCalled();
  }));

  it('saves datacard and creates observations when in "ultrasound" mode', fakeAsync(() => {
    component.isOpen = true;
    component.activity = getActivity();
    component.ngOnChanges({isOpen: new SimpleChange(false, true, true)});
    const dataCardSpy = spyOn(dataCardServiceStub, 'update').and.callThrough();
    const scanObservationSpy = spyOn(observationServiceStub, 'createUltrasoundObservations').and.callThrough();
    fixture.detectChanges();
    component.observationType = 'ultrasound';
    component.doSave();
    tick();
    expect(dataCardSpy).toHaveBeenCalled();
    expect(scanObservationSpy).toHaveBeenCalled();
  }));

  it('handles that active datacard could be undefined', fakeAsync(() => {
    component = new MeasurementsModalComponent({selectedDataCard$: of(undefined)} as unknown as DataCardService, observationServiceStub, noteServiceStub);
    component.ngOnInit();
    tick();
    expect(component.dataCard).toBeUndefined();
  }));

  it('should getMotherRows when obs type changes', () => {
    component.isOpen = true;
    component.ngOnChanges({observationType: new SimpleChange('regular', 'ultrasound', false)});
    expect(component.observationDataRegular?.fetuses?.length).toBe(1);
    expect(component.observationDataUltrasound?.fetuses?.length).toBe(1);
  });

  it('should handle closing when is Open changes', () => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: new SimpleChange(true, false, false)});
    expect(component.modalComponent?.isModalVisible).toBeFalse();
  });

  it('should do nothing if dialog is closed while already closed', () => {
    component.isOpen = true;
    expect(component.modalComponent?.isModalVisible).toBeFalse();
    component.ngOnChanges({isOpen: new SimpleChange(false, false, false)});
    expect(component.modalComponent?.isModalVisible).toBeFalse();
  });

  describe('Edit existing activity measurements', () => {
    const activityId = '1234-abcd';
    beforeEach(() => {
      component.activity = {
        id: activityId,
        observations: [
          {id: '1', activityReference: activityId, fetus: 'A', dateOfMeasurement: new Date(10, 8, 2019), type: 'fetus_weight_estimate', values: [{value: '2344', name: 'vægt', unit: 'g'}]},
          {id: '2', activityReference: activityId, fetus: 'A', dateOfMeasurement: new Date(10, 8, 2019), type: 'fetus_presentation', values: [{value: 'UK', name: 'Præsentation'}]}
        ],
        title: 'Vigtig Aktivitet',
        status: 'finished'
      };
    });

    it('should initiate observation  data with ultrasound scannings', fakeAsync(async () => {
      const expectedDvpValue = 1337;
      observationServiceStub.clinicalObservationTable$ = of({mother: {rows: [{id: '2'}]}, fetuses: [{id: '1'}]});
      observationServiceStub.ultrasoundObservationTable$ = of({mother: {rows: [{id: '2', activityId, dvp: {value: expectedDvpValue}}]}, fetuses: [{id: '1'}]});
      await component.initObservationData();
      expect(component.observationDataUltrasound?.dvp).toBe(expectedDvpValue);
    }));

    it('should initiate observation  data with regular observations',  fakeAsync(async () => {
      const expectedMothersWeight = 78;
      observationServiceStub.clinicalObservationTable$ = of({mother: {rows: [{id: '2',  activityId, weight: {value: expectedMothersWeight, unit: 'kg'}}]}, fetuses: [{id: '1'}]});
      observationServiceStub.ultrasoundObservationTable$ = of({mother: {rows: [{id: '2'}]}, fetuses: [{id: '1'}]});
      await component.initObservationData();
      expect(component.observationDataRegular?.weight).toBe(expectedMothersWeight);
    }));
    it('should do nothing on handleClose if modal is already closed', () => {
      component.isOpen = false;
      component.handleClose();
      expect(component.isOpen).toBe(false);
    });
    it('should do nothing on handleClose if modal is not visible', () => {
      component.isOpen = true;
      component.modalComponent = {
        isModalVisible: true,
        setVisible(isVisible: boolean): boolean {
          return isVisible;
        }
      } as any;
      component.handleClose();
      expect(component.isOpen).toBe(true);
    });
    it('should add ng-invalid class on activity changed to undefined', () => {
      component.activity = undefined;
      component.modalComponent?.setVisible(true);
      component.activityChanged();
    });
    it('should not attempt to get an activity date if activity is undefined', () => {
      component.setFormValues(undefined);
      expect(component.measurementDate).toBeUndefined();
    });
    it('should take conductedtime if it exists activity is finished', () => {
      const conductedTime = new Date(2021, 6, 20);
      component.setFormValues({status: 'finished', conductedTime} as any);
      expect(component.measurementDate?.toDateString()).toBe(conductedTime.toDateString());
    });
    it('should return an empty observable when trying to save without a datacard', () => {
      component.dataCard = undefined;
      component.observationType = 'ultrasound';
      component.doSave();
    });
    it('should map to correct flow', () => {
      component.initObservationData();
      expect(component.observationDataUltrasound?.flow).toBe('abnormal');
      observationServiceStub.ultrasoundObservationTable$ = of({mother: {rows: [{activityId: '1234-abcd', id: '1', flow: 'normal'}]}} as ScanningObservationTable);
      component.initObservationData();
      expect(component.observationDataUltrasound?.flow).toBe('normal');
      observationServiceStub.ultrasoundObservationTable$ = of({mother: {rows: [{activityId: '1234-abcd', id: '1', flow: 'der følges op'}]}} as ScanningObservationTable);
      component.initObservationData();
      expect(component.observationDataUltrasound?.flow).toBe('tobecontinued');
      observationServiceStub.ultrasoundObservationTable$ = of({mother: {rows: [{activityId: '1234-abcd', id: '1', flow: 'ugyldig værdi'}]}} as ScanningObservationTable);
      component.initObservationData();
      expect(component.observationDataUltrasound?.flow).toBe(undefined);
    });
    it('should return true when data is initialized', () => {
      expect(component.isObservationDataInitialized()).toBeTrue();
    });
    it('should set offset date on uSound table data ', () => {
      const offsetDate = new Date(2010, 6,  10);
      component.dataCard = {
        expectedDateOfDeliverySection: {
          offsetDate
        }
      } as any;
      component.initObservationData();
      expect(component.observationDataUltrasound?.offsetDate?.toDateString())
        .toEqual(offsetDate.toDateString());
    });
    it('should be editing if there are observations', () => {
      component.activity = getActivity();
      component.activity.observations = [
        {id: '1'},
        {id: '2'}
      ] as any[];
      expect(component.isEditing()).toBeTrue();
    });
  });
});
