import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges, ViewChild } from '@angular/core';
import { ModalComponent } from '../../modals/modal/modal.component';
import { Activity, DataCard, Note, SaveRegularObservationsRequest, SaveUltrasoundObservationsRequest } from '@nspop/gm-web-facade-api';
import { FormComponent } from '../../confirm-navigation-if-dirty.framework';
import { NgForm } from '@angular/forms';
import { copy } from '../../../utils/copy';
import { combineLatest, Observable, of } from 'rxjs';
import { concatMap, map } from 'rxjs/operators';
import { DataCardWrapper } from '../../model-wrappers/dataCard.wrapper';
import { ObservationService } from '../../api/observation.service';
import { DataCardService } from '../../api/data-card.service';
import { NoteService } from '../../api/note.service';
import { AppError } from '../../services/error.service';
import ObservationTableUtil from '../../../utils/observation-table.util';

export type measurementsModalEvent = {
  isOpen: boolean;
  observationType: ObservationFormType;
  allowTypeChange: boolean;
  activity?: Activity;
  allowActivityChange: boolean;
};

export type ObservationFormType = 'regular' | 'ultrasound';

@Component({
  selector: 'app-measurements-modal',
  templateUrl: './measurements-modal.component.html',
  styleUrls: [ './measurements-modal.component.scss' ]
})
export class MeasurementsModalComponent extends FormComponent implements OnChanges, OnInit {

  public note: Note = { content: '' };
  public isSaving = false;
  public dataCard?: DataCardWrapper;
  public observationDataRegular?: SaveRegularObservationsRequest | undefined;
  public observationDataUltrasound?: SaveUltrasoundObservationsRequest | undefined;
  public measurementDate: Date | undefined;

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @ViewChild('form') public form: NgForm | undefined;

  @Input() public isOpen = false;
  @Input() public allowTypeChange = true;
  @Input() public activity: Activity | undefined = undefined;
  @Input() public allowActivityChange = true;
  @Input() public observationType: ObservationFormType = 'regular';

  @Output() public modalClosed: EventEmitter<void> = new EventEmitter();
  @Output() public save: EventEmitter<void> = new EventEmitter();

  private dataCardId = '';
  private dataCardVersion = '';
  private ultrasoundActivityKeywords: string[] = [ 'scanning', 'skanning', 'ultralyd', 'ultralydsundersøgelse', 'sonograf' ];

  constructor(
    private dataCardService: DataCardService,
    private observationService: ObservationService,
    private noteService: NoteService
  ) {
    super();
  }

  ngOnInit(): void {
    this.dataCardService.selectedDataCard$.subscribe((dataCard?: DataCard) => {
      if (dataCard) {
        this.dataCardId = dataCard.documentUniqueId || '';
        this.dataCardVersion = dataCard.version || '';
        this.dataCard = new DataCardWrapper(copy(dataCard));
      }
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.isOpen) {
      if (changes.isOpen.currentValue) {
        this.initObservationData();
        this.modalComponent?.setVisible(true);
      } else {
        this.handleClose();
      }
    }
    if (this.isOpen && changes.observationType) {
      this.initObservationData();
    }
  }

  handleClose(): void {
    if (!this.isOpen) {
      return;
    }

    this.modalComponent?.setVisible(false);

    if (this.modalComponent?.isModalVisible) {
      return;
    }
    this.modalClosed.emit();

    this.clearObservationData();
  }

  activityChanged(): void {
    const activitySelector = document.getElementById('activitySelector');
    this.setFormValues(this.activity);
    if (activitySelector) {
      if (!this.activity && this.modalComponent && this.modalComponent.isModalVisible) {
        activitySelector.classList.add('ng-invalid');
      }
      activitySelector.classList.remove('ng-invalid');
    }
  }

  setFormValues(activity: Activity | undefined): void {
    if (!activity) {
      return;
    }

    function getDateFromActivity(selectedActivity: Activity | undefined): Date | undefined {
      if (!selectedActivity) {
        return;
      }
      switch (selectedActivity.status) {
        case 'finished':
          if (selectedActivity.conductedTime) {
            return selectedActivity.conductedTime;
          } else {
            return selectedActivity.plannedTime?.from;
          }
        default:
          return new Date();
      }
    }

    this.note = activity?.note ?? { content: '' };
    this.measurementDate = getDateFromActivity(activity);
  }

  doSave(): void {
    if (this.form?.valid) {
      if (!this.activity) {
        const activitySelector = document.getElementById('activitySelector');
        activitySelector?.scrollIntoView({ behavior: 'smooth', block: 'center' });
        return;
      }
      const activityId = this.activity?.id;
      this.setMeasurementDate(this.measurementDate);
      this.setActivityId(activityId);
      this.isSaving = true;
      const save = (this.observationType === 'ultrasound') ? this.saveUltrasound() : this.saveRegular();
      save.pipe(
          concatMap(() => this.noteService.createOrUpdateNote(activityId, this.note, this.note.date)))
        .subscribe(() => {
        }, (error => {
          throw new AppError(error?.errorMessage, error);
        }), async () => {
          await this.dataCardService.refresh();
          this.isSaving = false;
          this.handleClose();
        });
    } else {
      this.form?.resetForm(this.form?.form.value); // We make a fake reset, so the form is not seen as 'submitted'
      this.form?.form.markAsDirty(); // (Ensuring we still get prompted that we are going to discard changes if we try to leave the page)
      return;
    }
  }

  async initObservationData(): Promise<void> {
    if (this.activity?.observations?.length) {
      this.setMeasurementEditModel();
    } else {
      this.setTypeChangeAllowed(true);
      this.observationDataRegular = {
        fetuses: [ ...new Array(this.dataCard?.fetusCount || 1) ].map((x, index) => ({ id: this.getNthLetter(index) }))
      };
      this.observationDataUltrasound = {
        fetuses: [ ...new Array(this.dataCard?.fetusCount || 1) ].map((x, index) => ({ id: this.getNthLetter(index) }))
      };
      const title = this.activity?.title ? this.activity.title : '';
      this.observationType = this.isUltrasoundFromTitle(title) ? 'ultrasound' : this.observationType;
    }

    this.observationDataRegular = copy(this.observationDataRegular);
    this.observationDataUltrasound = copy(this.observationDataUltrasound);

    this.setFormValues(this.activity);

    this.form?.form.markAsPristine();
    this.form?.form.markAsUntouched();
  }

  isObservationDataInitialized(): boolean {
    return !!this.observationDataRegular && !!this.observationDataUltrasound;
  }

  getNthLetter(position: number): string {
    return String.fromCharCode('ABCDEFGHIJ'.charCodeAt(position));
  }

  isDatePickerDisabled(activity: Activity): boolean {
    if (!activity) {
      return true;
    }

    return activity.status === 'finished';
  }

  isEditing(): boolean {
    if (!this.activity?.observations) {
      return false;
    }
    return this.activity.observations.length > 0;
  }

  canChangeObservationType(): boolean {
    const isDirty = this.form && this.form?.dirty !== null && this.form.dirty;
    return this.getTypeChangeAllowed() && !isDirty;
  }

  private saveUltrasound(): Observable<void> {
    if (this.dataCard) {
      return this.dataCardService.update(this.dataCard.asDataCard, this.dataCardId, this.dataCardVersion)
        .pipe(
          concatMap(
            () => this.observationService.createUltrasoundObservations(this.observationDataUltrasound as SaveUltrasoundObservationsRequest))
        ) as unknown as Observable<void>;
    }
    return of();
  }

  private saveRegular(): Observable<void> {
    return this.observationService.createRegularObservations(this.observationDataRegular as SaveRegularObservationsRequest);
  }

  private setActivityId(activityId?: string): void {
    if (!this.observationDataRegular || !this.observationDataUltrasound) {
      return;
    }
    this.observationDataRegular.activityId = activityId;
    this.observationDataUltrasound.activityId = activityId;
  }

  private setMeasurementDate(measurementDate?: Date): void {
    if (!measurementDate) {
      return;
    }

    const offsetDate = this.dataCard?.expectedDateOfDeliverySection?.offsetDate;
    this.setMeasurementDateRegular(measurementDate, offsetDate);
    this.setMeasurementDateUltrasound(measurementDate, offsetDate);
  }

  private setMeasurementDateRegular(measurementDate: Date, offsetDate?: Date): void {
    if (!this.observationDataRegular) {
      return;
    }

    this.observationDataRegular.date = measurementDate;
    this.observationDataRegular.offsetDate = offsetDate;
  }

  private setMeasurementDateUltrasound(measurementDate: Date, offsetDate?: Date): void {
    if (!this.observationDataUltrasound) {
      return;
    }

    this.observationDataUltrasound.date = measurementDate;
    this.observationDataUltrasound.offsetDate = offsetDate;
  }

  private setMeasurementEditModel(): void {
    const offsetDate = this.dataCard?.expectedDateOfDeliverySection?.offsetDate;

    combineLatest([ this.observationService.ultrasoundObservationTable$, this.observationService.clinicalObservationTable$ ]).pipe(
      map(([ uSoundTable, clinicTable ]) => {
        const activityId = this.activity?.id;
        if (!activityId) {
          return;
        }

        if (ObservationTableUtil.hasScanningObservationsForActivityId({ activityId, table: uSoundTable })) {
          this.observationType = 'ultrasound';
          this.setTypeChangeAllowed(false);

          this.observationDataUltrasound = ObservationTableUtil.getObservationDataUltrasoundForActivity({ activityId, table: uSoundTable });

          if (offsetDate) {
            this.observationDataUltrasound.offsetDate = offsetDate;
          }

          if (!this.observationDataUltrasound?.fetuses || this.observationDataUltrasound?.fetuses?.length <= 0) {
            this.observationDataUltrasound.fetuses = [ ...new Array(this.dataCard?.fetusCount || 1) ]
              .map((x, index) => ({ id: this.getNthLetter(index) }));
          }
          return;
        }

        if (ObservationTableUtil.hasClinicalObservationsForActivityId({ activityId, table: clinicTable })) {
          this.observationType = 'regular';
          this.setTypeChangeAllowed(false);

          this.observationDataRegular = ObservationTableUtil.getObservationDataRegularForActivity({ activityId, table: clinicTable });

          if (offsetDate) {
            this.observationDataRegular.offsetDate = offsetDate;
          }

          if (!this.observationDataRegular?.fetuses || this.observationDataRegular?.fetuses?.length <= 0) {
            this.observationDataRegular.fetuses = [ ...new Array(this.dataCard?.fetusCount || 1) ]
              .map((x, index) => ({ id: this.getNthLetter(index) }));
          }
          return;
        }
      })
    ).subscribe();
  }

  private clearObservationData(): void {
    this.activity = undefined;
    this.observationDataRegular = undefined;
    this.observationDataUltrasound = undefined;
    this.measurementDate = undefined;
  }

  private setTypeChangeAllowed(isAllowed: boolean): void {
    this.allowTypeChange = isAllowed;
  }

  private getTypeChangeAllowed(): boolean {
    return this.allowTypeChange;
  }

  private isUltrasoundFromTitle(title: string): boolean {
    return title.includes('UL')
      || this.ultrasoundActivityKeywords.some(keyword => title.toLowerCase().includes(keyword));
  }
}
