import { ComponentFixture, TestBed } from '@angular/core/testing';
import { BloodPressureInputComponent } from './blood-pressure-input.component';
import {ReactiveFormsModule} from '@angular/forms';
import createSpyObj = jasmine.createSpyObj;
import {FocusMonitor} from '@angular/cdk/a11y';
import SpyObj = jasmine.SpyObj;

describe('BloodPressureInputComponent', () => {
  let component: BloodPressureInputComponent;
  let fixture: ComponentFixture<BloodPressureInputComponent>;

  let focusMonitorMock: SpyObj<FocusMonitor>;

  beforeEach(async () => {
    focusMonitorMock = createSpyObj<FocusMonitor>('FocusMonitor', ['focusVia']);

    await TestBed.configureTestingModule({
      imports: [ ReactiveFormsModule ],
      declarations: [ BloodPressureInputComponent ],
      providers: [{provide: FocusMonitor, useValue: focusMonitorMock}]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(BloodPressureInputComponent);
    component = fixture.componentInstance;
  });

  it('should create', () => {
    fixture.detectChanges();
    expect(component).toBeTruthy();
  });

  it('returns a valid blood pressure value if both systolic and diastolic is set', () => {
    fixture.detectChanges();
    const val = { systolic: 1, diastolic: 2};
    component.writeValue(val);
    expect(component.value).toEqual(val);
  });

  it('returns null if only one of systolic or diastolic is set', () => {
    fixture.detectChanges();

    const val1 = { systolic: 1, diastolic: null} as any;
    component.writeValue(val1);
    expect(component.value).toBe(null);

    const val2 = { diastolic: 1};
    component.writeValue(val2);
    expect(component.value).toBe(null);
  });

  it('returns null if it receives a falsy value', () => {
    fixture.detectChanges();
    component.writeValue(undefined);
    expect(component.value).toBe(null);
  });

  it('is in error state if touched and only one of systolic or diastolic is set', () => {
    component.touched = true;
    fixture.detectChanges();
    expect(component.errorState).toBe(false);

    const val1 = { systolic: 1};
    component.writeValue(val1);
    expect(component.errorState).toBe(true);

    const val2 = { diastolic: 1};
    component.writeValue(val2);
    expect(component.errorState).toBe(true);
  });

  it('is not in error state if not touched', () => {
    component.touched = false;
    fixture.detectChanges();
    expect(component.errorState).toBe(false);

    const val1 = { systolic: 1};
    component.writeValue(val1);
    expect(component.errorState).toBe(false);
  });

  it('disables individual parts when the input is disabled', () => {
    fixture.detectChanges();

    expect(component.disabled).toBe(false);

    const systolic = component.parts.get('systolic');
    expect(systolic).toBeDefined();
    expect(systolic?.enabled).toBe(true);

    component.disabled = true;

    expect(systolic?.enabled).toBe(false);

    component.disabled = false;

    expect(systolic?.enabled).toBe(true);
  });

  it('focuses next element when 3 characters are typed in', () => {
    fixture.detectChanges();

    const control = { value: '123', errors: false } as any;
    const element = {} as any;

    component.autoFocusNext(control, element);

    expect(focusMonitorMock.focusVia).toHaveBeenCalledTimes(1);
    expect(focusMonitorMock.focusVia).toHaveBeenCalledWith(element, 'program');

  });

  it('does not focus next element when less than 3 characters are typed in', () => {
    fixture.detectChanges();

    const control = { value: '12', errors: false } as any;
    const element = {} as any;

    component.autoFocusNext(control, element);

    expect(focusMonitorMock.focusVia).toHaveBeenCalledTimes(0);
  });

  it('focuses previous element when no characters are typed', () => {
    fixture.detectChanges();

    const control = { value: ''} as any;
    const element = {} as any;

    component.autoFocusPrev(control, element);

    expect(focusMonitorMock.focusVia).toHaveBeenCalledTimes(1);
    expect(focusMonitorMock.focusVia).toHaveBeenCalledWith(element, 'program');
  });

  it('does not focus previous element when there are more than 1 character typed in', () => {
    fixture.detectChanges();

    const control = { value: '1'} as any;
    const element = {} as any;

    component.autoFocusPrev(control, element);

    expect(focusMonitorMock.focusVia).toHaveBeenCalledTimes(0);
  });

  it('focuses right element when pressing navigation at the end of input', () => {
    fixture.detectChanges();

    const input = { selectionStart: 2, value: '12'} as any;
    const element = {} as any;

    component.handleRightInputNavigation(element, input);

    expect(focusMonitorMock.focusVia).toHaveBeenCalledTimes(1);
    expect(focusMonitorMock.focusVia).toHaveBeenCalledWith(element, 'program');
  });

  it('does not focus right element when pressing navigation not at the end of input', () => {
    fixture.detectChanges();

    const input = { selectionStart: 1, value: '12'} as any;
    const element = {} as any;

    component.handleRightInputNavigation(element, input);

    expect(focusMonitorMock.focusVia).toHaveBeenCalledTimes(0);
  });


  it('focuses left element when pressing navigation at the start of input', () => {
    fixture.detectChanges();

    const input = { selectionStart: 0, value: '12'} as any;
    const element = {} as any;

    component.handleLeftInputNavigation(element, input);

    expect(focusMonitorMock.focusVia).toHaveBeenCalledTimes(1);
    expect(focusMonitorMock.focusVia).toHaveBeenCalledWith(element, 'program');
  });

  it('does not focus left element when pressing navigation not at the start of input', () => {
    fixture.detectChanges();

    const input = { selectionStart: 1, value: '12'} as any;
    const element = {} as any;

    component.handleLeftInputNavigation(element, input);

    expect(focusMonitorMock.focusVia).toHaveBeenCalledTimes(0);
  });

  it('on focus in event it changes focused to true', () => {
    fixture.detectChanges();

    const focusEvent = {} as any;

    expect(component.focused).toBe(false);

    component.onFocusIn(focusEvent);

    expect(component.focused).toBe(true);
  });

  it('on focus out event it changes focused to false and touched to try', () => {
    fixture.detectChanges();

    const otherElement = document.createElement('div');

    const focusEvent = { relatedTarget: otherElement} as any;
    component.focused = true;

    expect(component.focused).toBe(true);
    expect(component.touched).toBe(false);

    component.onFocusOut(focusEvent);

    expect(component.focused).toBe(false);
    expect(component.touched).toBe(true);
  });

});
