import deepEqual from 'deep-equal';
import { DataCardWrapper } from '../../model-wrappers/dataCard.wrapper';
import { ObservationService } from '../../api/observation.service';
import { ObservationFormType } from '../measurements-modal/measurements-modal.component';
import { forwardRef, Component, OnInit, OnDestroy, Input } from '@angular/core';
import { NG_VALUE_ACCESSOR, ControlContainer, NgForm, ControlValueAccessor } from '@angular/forms';
import { DataCard, ScanningObservationTable, ScanningObservationTableFetusesInner, NumericValueWithUnit } from 'webfacade-api';

type Row = {
  activityId: string;
  date?: Date;
  gestationAge?: string;
  ulWeight?: string;
  weightDeviation?: string;
  fetusPresentation?: string;
  amnioticFluid?: string;
  flow?: string;
  cervix?: string;
  dvp?: string;
  conclusion?: string;
};

export const APP_MEASUREMENTS_ULTRASOUND_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MeasurementsUltrasoundComponent),
  multi: true
};

@Component({
  selector: 'app-measurements-ultrasound',
  templateUrl: './measurements-ultrasound.component.html',
  styleUrls: [ './measurements-ultrasound.component.scss', '../measurements-regular/tables.scss' ],
  providers: [ APP_MEASUREMENTS_ULTRASOUND_CONTROL_VALUE_ACCESSOR ],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class MeasurementsUltrasoundComponent implements ControlValueAccessor, OnInit, OnDestroy {
  public rows: Row[] = [];
  @Input() public showRelatedActivityModal: { isOpen: boolean; activityId?: string; observationType?: ObservationFormType } | undefined;
  @Input() public measurementsModal: { isOpen: boolean; observationType: ObservationFormType } | undefined;
  @Input() public saveListeners: (() => any)[] = [];

  private innerModel: DataCardWrapper | undefined;
  private dirtyAble = false;

  get value(): DataCardWrapper | undefined {
    return this.innerModel;
  }

  set value(value: DataCardWrapper | undefined) {
    if (!deepEqual(value?.asDataCard, this.innerModel?.asDataCard)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value?.asDataCard);
      }
      if (!this.dirtyAble && this.innerModel) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }
    }
  }

  constructor(public observationService: ObservationService) { }

  ngOnInit(): void {
    this.saveListeners.push(this.saveCallback);
    this.buildObservationTable();
  }

  ngOnDestroy(): void {
    this.saveListeners = this.saveListeners.splice(this.saveListeners.indexOf(this.saveCallback), 1);
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: DataCard): void {
    this.value = new DataCardWrapper(newValue);
  }

  showActivity(activityId: string): void {
    if (this.showRelatedActivityModal) {
      this.showRelatedActivityModal.activityId = activityId;
      this.showRelatedActivityModal.isOpen = true;
      this.showRelatedActivityModal.observationType = 'ultrasound';
    }
  }

  createMeasurements(): void {
    if (this.measurementsModal) {
      this.measurementsModal.observationType = 'ultrasound';
      this.measurementsModal.isOpen = true;
    }
  }

  private onChangeCallback: (newValue: DataCard | undefined) => any = () => {};

  private onTouchedCallback: () => any = () => {};

  private saveCallback = () => this.buildObservationTable();

  private buildObservationTable(): void {
    this.observationService.ultrasoundObservationTable$.subscribe((table: ScanningObservationTable | undefined) => {
      if (!table) { return; }

      this.rows = [];
      const activityGrouping: Record<string, ScanningObservationTable> = {};
      const activityIds: Set<string> = new Set();

      table.mother?.rows?.forEach(row => {
        if (row.activityId) {
          activityIds.add(row.activityId);
        }
      });

      table.fetuses?.forEach(fetus => fetus.rows?.forEach((row) => {
        if (row.activityId) {
          activityIds.add(row.activityId);
        }
      }));

      for (const activityId of activityIds) {
        const mother = { rows: table.mother?.rows?.filter(x => x.activityId === activityId) || [] };
        const fetuses = table.fetuses?.map(fetus => ({
          id: fetus.id,
          name: fetus.name,
          rows: fetus.rows?.filter((x) => x.activityId === activityId) || []
        }));
        activityGrouping[activityId] = { mother, fetuses };
      }
      Object.keys(activityGrouping).forEach(activityId => {
        let fetusWithLargestNumberOfRows: ScanningObservationTableFetusesInner | undefined;
        const data = activityGrouping[activityId];
        if (data?.fetuses && data.fetuses.length > 0) {
          fetusWithLargestNumberOfRows = data.fetuses?.reduce(
            (prevFetus, currentFetus) => {
              if (!prevFetus?.rows?.length) { return currentFetus; }
              if (!currentFetus?.rows?.length) { return prevFetus; }
              return prevFetus?.rows?.length > currentFetus?.rows?.length || 0 ? prevFetus : currentFetus;
            });
        }
        const numberOfRows = Math.max(data.mother?.rows?.length || 0, fetusWithLargestNumberOfRows?.rows?.length || 0);
        for (let i = 0; i < numberOfRows; i++) {
          const row: Row = {
            activityId,
            date: this.extractPropertyFromAny<Date>('date', data, i),
            gestationAge: this.extractPropertyFromAny<string>('gestationAge', data, i) || '-',
            ulWeight: this.mergeProperty('ulWeight', data.fetuses || [], i),
            weightDeviation: this.mergeProperty('weightDeviation', data.fetuses || [], i),
            fetusPresentation: this.mergeProperty('fetusPresentation', data.fetuses || [], i),
            amnioticFluid: this.displayNumericValue(data.mother?.rows?.[i]?.amnioticFluid),
            flow: data.mother?.rows?.[i]?.flow || '-',
            cervix: this.displayNumericValue(data.mother?.rows?.[i]?.cervix),
            dvp: this.displayNumericValue(data.mother?.rows?.[i]?.dvp)
          };
          this.rows.push(row);
        }
      });
    });
  }

  private extractPropertyFromAny<T>(property: string, table: ScanningObservationTable, i: number): T | undefined {
    if (table.mother?.rows) {
      if (Object.keys(table.mother?.rows[i] || {}).includes(property)) {
        const probablyResult = (table.mother?.rows[i] as any)[property];
        if (probablyResult) {
          return probablyResult as T;
        }
      }
    }
    let result: T | undefined;
    table.fetuses?.forEach((fetus) => {
      if ((Object.keys(fetus.rows?.[i] || {}) || []).includes(property)) {
        const probablyResult = (fetus.rows?.[i] as any)[property];
        if (probablyResult) {
          result = probablyResult;
          return;
        }
      }
    });
    return result;
  }

  private displayNumericValue(value?: NumericValueWithUnit): string {
    if (value && value.value) {
      return `${value.value} ${value.unit}`;
    }
    return '-';
  }

  private mergeProperty(property: string, feti: ScanningObservationTableFetusesInner[], i: number): string {
    const value = feti.map((fetus) => {
      // @ts-ignore
      const element = fetus?.rows?.[i]?.[property];
      if (!element) {
        return '-';
      }
      if (element.value && element.unit) {
        return this.displayNumericValue(element);
      }
      return element;
    });
    const filteredValues = value.filter(val => val !== '-');
    if (filteredValues.length > 1) {
      return filteredValues.reduce((a, b, index) => index === 0 ? b : a + ', ' + b);
    } else if (filteredValues.length === 1) {
      return filteredValues[0];
    } else {
      return '-';
    }
  }
}
